@@ -1,7 +1,7 @@
 #
 # rbfly - a library for RabbitMQ Streams using Python asyncio
 #
-# Copyright (C) 2021-2022 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2021-2023 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,26 @@ class RbFlyError(Exception):
     Basic RbFly exception.
     """
 
-class AMQPDecoderError(RbFlyError):
+class RbFlyBufferError(RbFlyError):
+    """
+    Buffer access error.
+
+    Raised when
+
+    - encoding of a message leads to buffer overflow
+    - and it is possible to recover from the error
+
+    .. seealso::
+
+       - :py:class:`~rbfly.error.AMQPDecoderError`
+    """
+
+class AMQPError(RbFlyError):
+    """
+    AMQP error.
+    """
+
+class AMQPDecoderError(AMQPError):
     """
     AMQP binary format decoder error.
     """
