@@ -1,7 +1,7 @@
 #
 # rbfly - a library for RabbitMQ Streams using Python asyncio
 #
-# Copyright (C) 2021-2022 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2021-2023 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,12 +26,11 @@ import typing as tp
 from functools import partial
 
 from . import const
-from ._codec import FrameDecoder, Encoder, decode_publish_confirm, \
+from ._codec import FrameDecoder, FrameEncoder, decode_publish_confirm, \
     decode_messages
 from .offset import Offset, OffsetType, SIMPLE_OFFSET
 from .types import PublishError
 
-FMT_FRAME_SIZE = struct.Struct('>I')
 FMT_ARRAY_LEN = struct.Struct('>i')
 FMT_STRING_LEN = struct.Struct('>h')
 
@@ -54,7 +53,6 @@ FMT_MESSAGE_ID = struct.Struct('>Q')
 FMT_REQUEST_RESPONSE = struct.Struct('>IH')
 FMT_REQUEST = struct.Struct('>HHI')
 
-LEN_FRAME_SIZE = FMT_FRAME_SIZE.size
 LEN_ARRAY_LEN = FMT_ARRAY_LEN.size
 LEN_HEADER = FMT_HEADER.size
 LEN_REQUEST_RESPONSE = FMT_REQUEST_RESPONSE.size
