@@ -1,7 +1,7 @@
 #
 # rbfly - a library for RabbitMQ Streams using Python asyncio
 #
-# Copyright (C) 2021-2022 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2021-2023 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -148,7 +148,7 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
 
         self.transport = Option[asyncio.Transport]()
         self.decoder = codec.FrameDecoder()
-        self.encoder = codec.Encoder(self.frame_size)
+        self.encoder = codec.FrameEncoder(self.frame_size)
 
         self._loop = asyncio.get_running_loop()
         self._requests: Requests = {}
@@ -298,7 +298,7 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
             message_id: int,
             *messages: MessageCtx | bytes,
             amqp: bool=True
-        ) -> tp.Any:
+    ) -> int:
         """
         Publish multiple messages to RabbitMQ stream.
 
@@ -315,17 +315,22 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         # publish message is sent, rabbitmq streams broker sends no
         # response and the publishing task waits forever
         if not messages:
-            return
+            return 0
+
+        count, frame = self.encoder.encode_publish(
+            publisher_id, message_id, *messages, amqp=amqp
+        )
+        assert count >= 0
+        if count == 0:
+            raise ValueError('Message data too long')
 
         task = self._loop.create_future()
-        pm = SentMessages(task, message_id, message_id + len(messages) - 1)
+        pm = SentMessages(task, message_id, message_id + count - 1)
         self._published_messages[publisher_id] = pm
-
-        frame = self.encoder.encode_publish(publisher_id, message_id, *messages, amqp=amqp)
         self.send_frame(frame)
 
-        assert len(messages) > 0
-        return await task
+        await task
+        return count
 
     @retry(ProtocolError, predicate=is_stream_na, retry_after=1)
     async def subscribe(
@@ -354,9 +359,9 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         )
 
         task = self._loop.create_future()
-        data = codec.encode_subscribe(sid, stream, offset, const.DEFAULT_CREDIT)
+        data = codec.encode_subscribe(sid, stream, offset, const.INITIAL_CREDIT)
 
-        queue = MessageQueue(const.DEFAULT_CREDIT, const.QUEUE_SIZE_THRESHOLD)
+        queue = MessageQueue(const.INITIAL_CREDIT, const.QUEUE_MAX_SIZE)
         self._subscriptions[sid] = ReceivedMessages(queue, value, amqp)
         return await self.send_request(const.KEY_SUBSCRIBE, data)
 
@@ -371,9 +376,9 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
 
         # credit of the subscription is usually renewed in
         # process_message_delivery method; it might not happen there if
-        # message queue threshold is breached, so check if it needs to be
-        # updated before we start waiting for new messages
-        if queue.needs_credit:
+        # message queue is full, so check if it needs to be updated before
+        # we start waiting for new messages
+        if queue.empty():
             self.renew_credit(subscription_id, queue)
 
         await queue.wait()
@@ -519,7 +524,7 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         :param credit: Amount of credit to send.
         """
         data = codec.encode_credit(subscription_id, credit)
-        self.send_frame(data)
+        self.send_data(data)
 
     def store_offset(self, stream: str, reference: str, offset: Offset) -> None:
         """
@@ -530,7 +535,7 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         :param offset: RabbitMQ Streams offset specification.
         """
         data = codec.encode_store_offset(stream, reference, offset)
-        self.send_frame(data)
+        self.send_data(data)
 
     def connection_made(self, transport: asyncio.BaseTransport) -> None:
         self.transport = Option(tp.cast(asyncio.Transport, transport))
@@ -582,7 +587,7 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
                 'sending heartbeat, time since last response: {:.1f}'
                 .format(delta)
             )
-            self.send_frame(codec.HEARTBEAT)
+            self.send_data(codec.HEARTBEAT)
             self._heartbeat_sent = True
 
             assert self._heartbeat_sent
@@ -611,13 +616,13 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
 
     def renew_credit(self, subscription_id: int, queue: MessageQueue) -> None:
         """
-        Renew credit of RabbitMQ stream subscription if necessary.
+        Renew credit of RabbitMQ stream subscription.
 
         :param subscription_id: RabbitMQ stream subscription id.
-        :param queue: RabbitMQ stream message queue.
+        :param queue: Message queue for a RabbitMQ stream.
         """
-        self.send_credit(subscription_id, queue.default_credit)
-        queue.reset_credit()
+        credit = queue.inc_credit()
+        self.send_credit(subscription_id, credit)
 
     def data_received(self, chunk: bytes) -> None:
         self._timestamp = self._loop.time()
@@ -634,7 +639,7 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
                 # simply wait for a heartbeat from rabbitmq streams broker
                 # and respond with another heartbeat; see also
                 # schedule_timestamp_callback method
-                self.send_frame(codec.HEARTBEAT)
+                self.send_data(codec.HEARTBEAT)
                 logger.info('heartbeat frame sent')
 
             elif key == const.KEY_TUNE:
@@ -664,15 +669,18 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
             else:
                 logger.warning('unknown key; key=0x{:04x}'.format(key))
 
+    def send_data(self, data: bytes) -> None:
+        """
+        Send RabbitMQ streams frame to broker.
+        """
+        frame = self.encoder.encode(data)
+        self.send_frame(frame)
+
     def send_frame(self, data: bytes) -> None:
         """
         Send RabbitMQ streams frame to broker.
         """
         transport = self.transport.value
-        n = len(data)
-        if n > self.frame_size:
-            raise ProtocolError(0x0e, 'Frame too large')
-        transport.write(codec.FMT_FRAME_SIZE.pack(n))
         transport.write(data)
 
     async def send_request(self, key: int, data: bytes) -> tp.Any:
@@ -680,7 +688,7 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         correlation_id = self._correlation_id
 
         request_data = codec.create_request(key, correlation_id, data)
-        self.send_frame(request_data)
+        self.send_data(request_data)
         task = self._requests[correlation_id] = self._loop.create_future()
 
         if __debug__:
@@ -827,11 +835,11 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         frame_size, heartbeat = codec.FMT_TUNE.unpack_from(data, start)[-2:]
         self.frame_size = frame_size
         self.heartbeat = heartbeat
-        self.encoder = codec.Encoder(frame_size)
+        self.encoder = codec.FrameEncoder(frame_size)
 
         # tune request is sent by rabbitmq streams broker; respond
         # to the request with another tune request (not a response)
-        self.send_frame(data[start:start + codec.FMT_TUNE.size])
+        self.send_data(data[start:start + codec.FMT_TUNE.size])
         task = self._waiters.pop(const.KEY_TUNE)
         task.set_result(None)
         logger.info(
