@@ -1,7 +1,7 @@
 #
 # rbfly - a library for RabbitMQ Streams using Python asyncio
 #
-# Copyright (C) 2021-2022 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2021-2023 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@ from datetime import datetime, timezone
 
 from rbfly.amqp._message import MessageCtx, encode_amqp, decode_amqp
 from rbfly.types import Symbol
-from rbfly.error import AMQPDecoderError
+from rbfly.error import RbFlyBufferError, AMQPDecoderError
 
 import pytest
 
@@ -199,12 +199,14 @@ DATA_ANNOTATED = (
     ),
 )
 
-DATA_INVALID = (
-    # minimum 4 bytes expected
-    b'\x00Sw',
-
+DATA_BUFFER_INVALID = (
     # uint32 missing one byte
     b'\x00Sw\x71\x80\x00\x00',
+)
+
+DATA_AMQP_INVALID = (
+    # minimum 4 bytes expected
+    b'\x00Sw',
 
     # string/byte string: expected buffer size is one more byte
     b'\x00Sw\xa1\x06abcde',
@@ -225,14 +227,6 @@ def test_encode(message: MessageCtx, expected: bytes) -> None:
     size = encode_amqp(msg_buffer, message)
     assert bytes(msg_buffer[:size]) == expected
 
-def test_encode_binary_invalid() -> None:
-    """
-    Test error on encoding too long binary AMQP messages.
-    """
-    msg_buffer = bytearray(1024)
-    with pytest.raises(ValueError):
-        encode_amqp(msg_buffer, MessageCtx(b'a' * 2 ** 32))
-
 @pytest.mark.parametrize('message, data', DATA + DATA_PARSED)
 def test_decode(message: MessageCtx, data: bytes) -> None:
     """
@@ -256,12 +250,47 @@ def test_message_repr(message: MessageCtx, expected: str) -> None:
     """
     assert repr(message) == expected
 
-@pytest.mark.parametrize('data', DATA_INVALID)
-def test_decode_invalid_size(data: bytes) -> None:
+@pytest.mark.parametrize('data', DATA_AMQP_INVALID)
+def test_decode_invalid_amqp(data: bytes) -> None:
     """
-    Test decoding AMQP data when declared size and buffer do not agree.
+    Test decoding invalid AMQP data.
     """
     with pytest.raises(AMQPDecoderError):
         decode_amqp(data)
 
+# buffer related tests
+
+def test_encode_amqp_string_too_long() -> None:
+    """
+    Test error when encoding too long string message.
+    """
+    msg_buffer = bytearray(1024)
+    with pytest.raises(RbFlyBufferError):
+        encode_amqp(msg_buffer, MessageCtx(b'a' * 2 ** 32))
+
+def test_encode_amqp_list_too_long() -> None:
+    """
+    Test error when encoding too long list message.
+    """
+    msg_buffer = bytearray(1024)
+    with pytest.raises(RbFlyBufferError):
+        encode_amqp(msg_buffer, MessageCtx(list(range(204))))
+
+def test_encode_amqp_dict_too_long() -> None:
+    """
+    Test error when encoding too long dictionary message.
+    """
+    msg_buffer = bytearray(1024)
+    with pytest.raises(RbFlyBufferError):
+        data = dict(zip(range(102), range(102)))
+        encode_amqp(msg_buffer, MessageCtx(data))  # type: ignore[arg-type]
+
+@pytest.mark.parametrize('data', DATA_BUFFER_INVALID)
+def test_decode_invalid_buffer(data: bytes) -> None:
+    """
+    Test decoding AMQP data with buffer compromised.
+    """
+    with pytest.raises(RbFlyBufferError):
+        decode_amqp(data)
+
 # vim: sw=4:et:ai
