@@ -1,7 +1,7 @@
 #
 # rbfly - a library for RabbitMQ Streams using Python asyncio
 #
-# Copyright (C) 2021-2022 by Artur Wroblewski <wrobell@riseup.net>
+# Copyright (C) 2021-2023 by Artur Wroblewski <wrobell@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ import typing as tp
 from collections import deque
 
 from rbfly.amqp import MessageCtx
+from rbfly.error import RbFlyBufferError
 from rbfly.streams import codec
 from rbfly.streams._mqueue import MessageQueue
 from rbfly.streams.offset import Offset
@@ -163,34 +164,137 @@ def test_encode_publish_single() -> None:
     """
     Test encoding single published message.
     """
-    encoder = codec.Encoder(1024)
-    data = [MessageCtx(b'123')]
-    result = encoder.encode_publish(5, 10, *data)
+    encoder = codec.FrameEncoder(1024)
+    messages = [MessageCtx(b'123')]
+    count, data = encoder.encode_publish(5, 10, *messages)
 
-    expected = b'\x00\x02\x00\x01\x05\x00\x00\x00\x01' \
+    expected = b'\x00\x00\x00\x1d' \
+        + b'\x00\x02\x00\x01\x05\x00\x00\x00\x01' \
         + b'\x00\x00\x00\x00\x00\x00\x00\x0a\x00\x00\x00\x08\x00Su\xa0\x03123'
-    assert result == expected
+    assert count == 1
+    assert data == expected
 
 def test_encode_publish_multi() -> None:
     """
     Test encoding multiple published messages.
     """
-    encoder = codec.Encoder(1024)
-    data = [MessageCtx(b'123') , MessageCtx(b'9876')]
-    result = encoder.encode_publish(5, 10, *data)
+    encoder = codec.FrameEncoder(1024)
+    messages = [MessageCtx(b'123') , MessageCtx(b'9876')]
+    count, data = encoder.encode_publish(5, 10, *messages)
 
-    expected = b'\x00\x02\x00\x01\x05\x00\x00\x00\x02' \
+    expected = b'\x00\x00\x002' \
+        + b'\x00\x02\x00\x01\x05\x00\x00\x00\x02' \
         + b'\x00\x00\x00\x00\x00\x00\x00\x0a\x00\x00\x00\x08\x00Su\xa0\x03123' \
         + b'\x00\x00\x00\x00\x00\x00\x00\x0b\x00\x00\x00\x09\x00Su\xa0\x049876'
 
-    assert result == expected
+    assert count == 2
+    assert data == expected
+
+def test_encode_publish_single_overflow() -> None:
+    """
+    Test overflow when encoding single message to publish.
+    """
+    encoder = codec.FrameEncoder(128)
+    messages = [MessageCtx(b'1' * 103)]  # at 102, no error
+    count, data = encoder.encode_publish(5, 10, *messages)
+    assert count == 0
+    # - there is still some data encoded, but count == 0
+    # - frame size
+    # - key publish, version, publisher id
+    # - num of messages
+    expected = b'\00\x00\x00\x09' \
+        + b'\x00\x02\x00\x01\x05' \
+        + b'\x00\x00\x00\x00'
+    assert data == expected
+
+def test_encode_publish_multi_overflow() -> None:
+    """
+    Test overflow when encoding multiple messages to publish.
+    """
+    encoder = codec.FrameEncoder(128)
+
+    # 5 messages, then no error
+    messages = [MessageCtx('b: {}'.format(i)) for i in range(6)]
+    count, data = encoder.encode_publish(3, 10, *messages)
+    assert count == 5
+
+    # - 114 bytes encoded in the frame
+    # - frame size
+    # - key publish, version, publisher id
+    # - num of messages
+    assert data.startswith(
+        b'\x00\x00\x00r'
+        + b'\x00\x02\x00\x01\x03'
+        + b'\x00\x00\x00\x05'
+    )
+    # check the last message
+    assert data.endswith(b'Sw\xa1\x04b: 4')
+
+    # 114 bytes plus 4 bytes for frame size
+    assert len(data) == 118
+
+def test_encode_publish_list_overflow() -> None:
+    """
+    Test overflow when encoding list message to publish.
+    """
+    encoder = codec.FrameEncoder(128)
+
+    messages = [
+        MessageCtx('b: ok'),
+        MessageCtx(list(range(32))),
+    ]
+    count, data = encoder.encode_publish(3, 10, *messages)
+    assert count == 1
+
+    # - 31 bytes encoded in the frame
+    # - frame size
+    # - key publish, version, publisher id
+    # - num of messages
+    assert data.startswith(
+        b'\x00\x00\x00\x1f'
+        + b'\x00\x02\x00\x01\x03'
+        + b'\x00\x00\x00\x01'
+    )
+    # check the last message
+    assert data.endswith(b'Sw\xa1\x05b: ok')
+
+    # 31 bytes plus 4 bytes for frame size
+    assert len(data) == 35
+
+def test_encode_publish_dict_overflow() -> None:
+    """
+    Test overflow when encoding dictionary message to publish.
+    """
+    encoder = codec.FrameEncoder(128)
+
+    messages = [
+        MessageCtx('b: ok'),
+        MessageCtx(dict(zip(range(102), range(102)))),
+    ]
+    count, data = encoder.encode_publish(3, 10, *messages)
+    assert count == 1
+
+    # - 31 bytes encoded in the frame
+    # - frame size
+    # - key publish, version, publisher id
+    # - num of messages
+    assert data.startswith(
+        b'\x00\x00\x00\x1f'
+        + b'\x00\x02\x00\x01\x03'
+        + b'\x00\x00\x00\x01'
+    )
+    # check the last message
+    assert data.endswith(b'Sw\xa1\x05b: ok')
+
+    # 31 bytes plus 4 bytes for frame size
+    assert len(data) == 35
 
 @pytest.mark.parametrize('data, offset, expected', DELIVERED_DATA)
 def test_decode_delivered_message(
         data: bytes,
         offset: tp.Optional[int],
         expected: list[MessageCtx]
-    ) -> None:
+) -> None:
     """
     Test decoding delivered messages.
     """
@@ -216,20 +320,33 @@ def test_decode_publish_error() -> None:
 #
 # protocol frame encoding and decoding
 #
-def test_send_frame() -> None:
+def test_send_data() -> None:
     """
-    Test sending RabbitMQ streams frame with asyncio transport.
+    Test sending data as RabbitMQ Streams protocol frame with asyncio
+    transport.
     """
     transport = mock.MagicMock()
     with mock.patch('asyncio.get_running_loop'):
         protocol = RabbitMQStreamsProtocol()
         protocol.connection_made(transport)
 
-        protocol.send_frame(b'54321')
+        protocol.send_data(b'54321')
         calls = transport.write.call_args_list
+        assert calls == [mock.call(b'\x00\x00\x00\x0554321')]
 
-        # size of the frame is sent first, then its data
-        assert calls == [mock.call(b'\x00\x00\x00\x05'), mock.call(b'54321')]
+def test_send_frame() -> None:
+    """
+    Test sending data as RabbitMQ Streams protocol frame with asyncio
+    transport.
+    """
+    transport = mock.MagicMock()
+    with mock.patch('asyncio.get_running_loop'):
+        protocol = RabbitMQStreamsProtocol()
+        protocol.connection_made(transport)
+
+        protocol.send_frame(b'\x00\x00\x00\x0554321')
+        calls = transport.write.call_args_list
+        assert calls == [mock.call(b'\x00\x00\x00\x0554321')]
 
 def test_frame_on_empty() -> None:
     """
@@ -283,7 +400,6 @@ def test_frame_incomplete() -> None:
     assert next(it, None) is None
     assert decoder.data == b''
 
-
 def create_frame(data: bytes, n: tp.Optional[int]=None) -> bytes:
     """
     Utility function to create RabbitMQ Streams protocol frame.
